@@ -274,6 +274,9 @@ class Query(object):
         +==============================================
 
     """
+
+    SO_SPLIT_PATTERN = re.compile("\s*(asc|desc)\s*", re.I)
+
     def __init__(self, model, service=None, validate=True, root=None):
         """
         Construct a new Query
@@ -400,10 +403,57 @@ class Query(object):
             con = obj.add_constraint(**args)
             if not con:
                 raise ConstraintError("error adding constraint with args: " + args)
+
+        def group(iterator, count):
+            itr = iter(iterator)
+            while True:
+                yield tuple([itr.next() for i in range(count)])
+
+        if q.getAttribute('sortOrder') is not None:
+            sos = Query.SO_SPLIT_PATTERN.split(q.getAttribute('sortOrder'))
+            if len(sos) == 1:
+                if sos[0] in obj.views: # Be tolerant of irrelevant sort-orders
+                    obj.add_sort_order(sos[0])
+            else:
+                sos.pop() # Get rid of empty string at end
+                for path, direction in group(sos, 2):
+                    if path in obj.views: # Be tolerant of irrelevant so.
+                        obj.add_sort_order(path, direction)
+
+        if q.getAttribute('constraintLogic') is not None:
+            logic = q.getAttribute('constraintLogic')
+            used_codes = set(obj.constraint_dict.keys())
+            logic_codes = set(re.split("\s*(?:and|or|\(|\))\s*", logic, flags=re.I))
+            if "" in logic_codes:
+                logic_codes.remove("")
+            irrelevant_codes = logic_codes - used_codes
+            for c in irrelevant_codes:
+                logic = re.sub("((and|or)\s+)?" + c + "(\s+(and|or))?", "", logic)
+            # Remove empty groups
+            logic = re.sub("\(\s*\)", "", logic)
+            # Remove trailing and leading opertators
+            logic = re.sub("\s*(and|or)\s*$", "", logic, flags=re.I)
+            logic = re.sub("^\s*(and|or)\s*", "", logic, flags=re.I)
+            ops = ["and", "or"]
+            for left, right in [(x, y) for x in ops for y in ops]:
+                repl = left if left == right else "and"
+                logic = re.sub(left + "\s*" + right, repl, logic, flags=re.I)
+                if left != right:
+                    logic = re.sub(right + "\s*" + left, repl, logic, flags=re.I)
+            logic = logic.strip().lstrip()
+            try: 
+                if len(logic) > 0:
+                    obj.set_logic(logic)
+            except:
+                raise Exception("Error parsing " + q.getAttribute('constraintLogic') + " => " + repr(logic) + " with views: " + repr(used_codes))
+
         obj.verify()        
 
         return obj
 
+    def __str__(self):
+        return self.to_xml()
+
     def verify(self):
         """
         Validate the query
@@ -792,13 +842,19 @@ class Query(object):
         This returns the up to date logic expression. The default
         value is the representation of all coded constraints and'ed together.
 
+        If the logic is empty and there are no constraints, returns an 
+        empty string.
+
         The LogicGroup object stringifies to a string that can be parsed to 
         obtain itself (eg: "A and (B or C or D)").
 
         @rtype: L{intermine.constraints.LogicGroup}
         """
         if self._logic is None:
-            return reduce(lambda x, y: x+y, self.coded_constraints)
+            if len(self.coded_constraints) > 0:
+                return reduce(lambda x, y: x+y, self.coded_constraints)
+            else: 
+                return ""
         else:
             return self._logic
 
@@ -1173,6 +1229,7 @@ class Query(object):
         """
         n = self.to_Node()
         return n.toxml()
+
     def to_formatted_xml(self):
         """
         Return a readable XML serialisation of the query
